<%@ page contentType="text/javascript" %>
<%@ taglib prefix="formfactory" uri="http://www.jahia.org/formfactory/functions" %>
<%--@elvariable id="renderContext" type="org.jahia.services.render.RenderContext"--%>

(function () {
    angular
        .module('formFactory')
        .directive('jexperienceConsent', jExperienceConsentDirective);
    // Add marketingFactoryConsent directive to support old declared form elements
    angular
        .module('formFactory')
        .directive('marketingFactoryConsent', jExperienceConsentDirective);

    jExperienceConsentDirective.$inject = ['ffTemplateResolver' ];

    function jExperienceConsentDirective(ffTemplateResolver) {

        return {
            restrict: 'E',
            require: ['^ffController'],
            controller: jExperienceConsentController,
            controllerAs: 'mfc',
            bindToController : true,
            link : function ($scope, el, attrs, controller) {
                $scope.viewType = attrs.viewType;
            },
            templateUrl: function (el, attrs) {
                return ffTemplateResolver.resolveTemplatePath('${formfactory:addFormFactoryModulePath('/formfactory-definitions/jexperience-consent', renderContext)}', attrs.viewType);
            }
        };

    }

    jExperienceConsentController.$inject = ['$log', '$http', 'contextualData', '$scope'];

    function jExperienceConsentController($log, $http, contextualData, $scope) {
        var vm = this;
        vm.consentTypes = [];
        vm.activeContentLanguage = {};

        vm.isConsentAvailable = isConsentAvailable;
        vm.changeConsent = changeConsent;

        vm.$onInit = function() {
            vm.language = contextualData.language ? contextualData.language : contextualData.locale;
            $scope.input.consentTitle = 'Loading...';
            $scope.input.consentDescription = 'Loading...';

            // Get all consent types
            $http.get('${url.context}${url.baseLive}${renderContext.site.path}.getConsentTypes.do').then(function (response) {
                vm.consentTypes = response.data.consentTypes;

                if ($scope.viewType !== 'designView') {
                    if ($scope.input.consentTypeId) {
                        vm.selectedConsentType = _getConsentType($scope.input.consentTypeId);

                        if (vm.selectedConsentType) {
                            $scope.input.consentTitle = vm.selectedConsentType.title[vm.language];
                            $scope.input.consentDescription = vm.selectedConsentType.description[vm.language];

                            if (vm.selectedConsentType.activated) {
                                var newConsentCallbackObject = {
                                    formName: $scope.form.$name,
                                    consentTypeId: $scope.input.consentTypeId,
                                    inputName: $scope.input.name,
                                    getConsentTypeEvent: _getConsentTypeEvent
                                };

                                window.consentCallbacks = window.consentCallbacks || [];
                                window.consentCallbacks.push(newConsentCallbackObject);

                                $log.info('[WEM - FF] mfffConsentInputDefinition - Initialized consent', vm.selectedConsentType);
                            }

                            // if wem is available register callback so we can set the value of the consent for the current profile
                            if (window.wem) {
                                window.wem._registerCallback(function () {
                                    if (window.digitalData) {
                                        var consentEntryName = window.digitalData.scope + '/' + $scope.input.consentTypeId;
                                        if (window.cxs && window.cxs.consents && window.cxs.consents[consentEntryName]) {
                                            if (window.cxs.consents[consentEntryName].status === 'GRANTED') {
                                                $scope.input.value = true;
                                            }
                                        }
                                    }
                                });
                            }
                        }
                    }
                }
            }, function (response) {
                $log.error('[WEM - FF] mfffConsentInputDefinition - Error retrieving consent types');
                $log.error(response);
            });

        };

        function changeConsent(newConsentTypeId) {
            if (newConsentTypeId) {
                vm.selectedConsentType = _getConsentType(newConsentTypeId);
                $scope.input.consentTitle = vm.selectedConsentType.title[vm.language];
                $scope.input.consentDescription = vm.selectedConsentType.description[vm.language];
            }
        }

        function isConsentAvailable() {
            if (!$scope.isLiveMode() && contextualData.isPreviewMode != 'true') {
                return true;
            }
            if (vm.selectedConsentType) {
                return vm.selectedConsentType.activated;
            } else {
                return false;
            }
        }

        function _getConsentType(consentTypeId) {
            if (!vm.consentTypes) {
                return null;
            }
            if (!consentTypeId) {
                return null;
            }
            var foundConsentType = null;
            angular.forEach(vm.consentTypes, function (consentType) {
                if (consentType.identifier === consentTypeId) {
                    foundConsentType = consentType;
                }
            });
            if (!foundConsentType) {
                $log.warn('[WEM - FF] mfffConsentInputDefinition - Could not find consent type for consent with id: '
                    + consentTypeId + ', maybe it was deleted?');
            }
            return foundConsentType;
        }

        function _getConsentTypeEvent(formData, formInfo, inputName, consentTypeId) {
            console.info('[WEM - FF] mfffConsentInputDefinition - preparing send event to Apache Unomi');

            var consentStatus = 'DENIED';
            if (formData.resultData[inputName] && formData.resultData[inputName] == true) {
                consentStatus = 'GRANTED';
            }

            var nowDate = new Date();
            var inTwoYearsDate = new Date();
            // 2 years expiration by default.
            inTwoYearsDate.setTime(nowDate.getTime() + 2 * 365 * 24 * 60 * 60 * 1000);

            return {
                scope: window.digitalData.scope,
                eventType: 'modifyConsent',
                source: {
                    itemType: 'page',
                    scope: window.digitalData.scope,
                    itemId: window.digitalData.page.pageInfo.pageID,
                    properties: window.digitalData.page
                },
                target: {
                    itemType: 'consent',
                    scope: window.digitalData.scope,
                    itemId: consentTypeId
                },
                properties: {
                    consent: {
                        scope : window.digitalData.scope,
                        typeIdentifier: consentTypeId,
                        status: consentStatus,
                        statusDate: nowDate.toISOString(),
                        revokeDate: inTwoYearsDate.toISOString()
                    }
                }
            }
        }
    }
})();
