<%@ page contentType="text/javascript" %>
<%--@elvariable id="renderContext" type="org.jahia.services.render.RenderContext"--%>

(function () {
    'use strict';

    var mfffPrefillService = function (contextualData, $q, $timeout, $http, $window, $interval) {
        var dataCache = null;
        var deferred = null;
        var typeMap = {
            "fcnt:inputDefinition": {
                type: ["string"],
                valueType: "single"
            },
            "fcnt:emailDefinition": {
                type: ["email"],
                valueType: "single"
            },
            "fcnt:passwordDefinition": {
                type: null,
                valueType: null
            },
            "fcnt:textAreaDefinition": {
                type: ["string"],
                valueType: "single"
            },
            "fcnt:fileUploadDefinition": {
                type: null,
                valueType: null
            },
            "fcnt:simpleDateDefinition": {
                type: ["date"],
                valueType: "single"
            },
            "fcnt:datePickerDefinition": {
                type: ["date"],
                valueType: "single"
            },
            "fcnt:ratingDefinition": {
                type: ["integer"],
                valueType: "single"
            },
            "fcnt:switchDefinition": {
                type: ["string", "boolean"],
                valueType: "single"
            },
            "fcnt:hiddenDefinition": {
                type: ["string", "boolean", "integer", "date", "email"],
                valueType: "single"
            },
            "fcnt:multipleCheckBoxesInlineDefinition": {
                type: ["string", "boolean"],
                valueType: "multiple"
            },
            "fcnt:multipleCheckBoxesDefinition": {
                type: ["string", "boolean"],
                valueType: "multiple"
            },
            "fcnt:multipleRadiosDefinition": {
                type: ["string", "boolean"],
                valueType: "single"
            },
            "fcnt:multipleRadiosInlineDefinition": {
                type: ["string", "boolean"],
                valueType: "single"
            },
            "fcnt:selectBasicDefinition": {
                type: ["string"],
                valueType: "single"
            },
            "fcnt:selectMultipleDefinition": {
                type: ["string"],
                valueType: "multiple"
            },
            "fcnt:countryListDefinition": {
                type: ["string"],
                valueType: "single"
            },
            "fcnt:matrixRadiosDefinition": {
                type: null,
                valueType: null
            },
            "fcnt:matrixCheckBoxesDefinition": {
                type: null,
                valueType: null
            },
            "fcnt:mfffConsentInputDefinition": {
                type: null,
                valueType: null
            }
        };

        this.getTypeMap = function() {
            return typeMap;
        };

        /**
         * MUST IMPLEMENT
         *
         * Gets data for prefill which must be an object:  { key: value, ... }
         * @returns {Promise}
         */
        this.getData = function() {
            if (deferred !== null) {
                return deferred;
            }

            deferred = $q.defer();

            if (dataCache !== null) {
                $timeout(function(resolve, reject) {
                    deferred.resolve(dataCache);
                }, 100);
                return deferred;
            }

            var self = this;
            var profileAcquired = false;
            //Start checking for cxs object on the window every 5 milliseconds
            var getCxsObject = $interval(function() {
                if ($window.cxs !== undefined && $window.cxs.profileId !== undefined) {
                    profileAcquired = true;
                    $interval.cancel(getCxsObject);

                    var profileId = $window.cxs.profileId;
                    $http({
                        url: contextualData.context + self.getProxyServletUrl() + "/cxs/profiles/" + profileId + "?time=" + Date.now(),
                        method: "GET"
                    }).then(function(data) {
                        //Process keys to fit desired format
                        dataCache = transformData(data.data.properties);
                        deferred.resolve(dataCache);
                    }, function(error) {
                        console.log(error);
                        deferred.resolve({});
                    });

                }
            }, 5);

            //Cancel check for cxs if it takes too long
            $timeout(function() {
                $interval.cancel(getCxsObject);
                deferred.resolve({});
                if (!profileAcquired) {
                    console.info("Failed to get profileId from window.cxs object or the object itself. Your server is slow, fix it!");
                }
            }, (wem && wem.timeoutInMilliseconds && wem.timeoutInMilliseconds != -1)?wem.timeoutInMilliseconds:3000);

            return deferred;
        };

        this.getProxyServletUrl = function() {
            return '/modules/jexperience/proxy/' + '${renderContext.site.siteKey}';
        };

        this.clearErrors = function(event) {
            var inputDiv = $(event.currentTarget).closest(".panel").find(".mfffSelectHolder");
            inputDiv.removeClass('has-error');
            inputDiv.find('.help-block').addClass('hide');
        };

        function transformData(data) {
            <%--console.log("Get from unomi", data);--%>
            $window.profileProperties = data;
            contextualData["mfProfileProperties"] = data;
            return data;
        }
    };

    angular.module('formFactory')
        .service('mfffPrefillService', ['contextualData',
            '$q', '$timeout', '$http', '$window', '$interval', mfffPrefillService]);
})();
